package com.kalyshev.yan.cabinet.controller;

import com.kalyshev.yan.cabinet.service.CabinetService;
import com.kalyshev.yan.computer.controller.ComputerDto;
import com.kalyshev.yan.configuration.OpenAPI30Configuration;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@RestController
@RequestMapping(OpenAPI30Configuration.API_PREFIX + "/cabinet")
public class CabinetController {
    private final CabinetService cabinetService;
    public CabinetController(CabinetService cabinetService) {
        this.cabinetService = cabinetService;
    }
    @GetMapping("/{id}")
    public CabinetDto getCabinet(@PathVariable Long id) {
        return new CabinetDto(cabinetService.findCabinet(id));
    }
    @GetMapping("/")
    public List<CabinetDto> getCabinets() {
        return cabinetService.findAllCabinets().stream()
                .map(CabinetDto::new)
                .toList();
    }
    @GetMapping("/{id}/computers")
    public List<ComputerDto> getCabinetComputers(@PathVariable Long id) {
        return cabinetService.listComputersFromCabinet(id).stream()
                .map(ComputerDto::new)
                .toList();
    }
    @GetMapping("/filter")
    public List<CabinetDto> getFilteredCabinets(@RequestParam(value = "id", required = false) Long id,
                                         @RequestParam(value = "number", required = false) String number) {
        return cabinetService.findFilteredCabinets(id, number).stream()
                .map(CabinetDto::new)
                .toList();
    }
    @PostMapping("/")
    public CabinetDto createCabinet(@RequestBody @Valid CabinetDto cabinetDto) {
        return new CabinetDto(cabinetService.addCabinet(cabinetDto.getNumber()));
    }
    @PostMapping("/{id}/computer")
    public void createCabinetComputer(@PathVariable Long id,
                                          @RequestParam("computerId") Long computerId) {
        cabinetService.addComputerToCabinet(computerId, id);
    }
    @PutMapping("/{id}")
    public CabinetDto updateCabinet(@PathVariable Long id,
                                    @RequestBody @Valid CabinetDto cabinetDto) {
        return new CabinetDto(cabinetService.updateCabinet(id, cabinetDto.getNumber()));
    }
    @DeleteMapping("/{id}")
    public CabinetDto deleteCabinet(@PathVariable Long id) {
        return new CabinetDto(cabinetService.deleteCabinet(id));
    }
    @DeleteMapping("/{cabinetId}/computer")
    public void deleteCabinetComputer(@PathVariable Long cabinetId,
                                             @RequestParam("computerId") Long computerId) {
        cabinetService.deleteComputerFromCabinet(computerId, cabinetId);
    }
}