package com.kalyshev.yan;

import com.kalyshev.yan.cabinet.model.Cabinet;
import com.kalyshev.yan.cabinet.repository.CabinetNotFoundException;
import com.kalyshev.yan.cabinet.service.CabinetService;
import com.kalyshev.yan.computer.service.ComputerService;
import com.kalyshev.yan.monitor.service.MonitorService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.List;

@SpringBootTest
public class JpaCabinetTests {
    private static final Logger log = LoggerFactory.getLogger(JpaCabinetTests.class);
    @Autowired
    private ComputerService computerService;
    @Autowired
    private MonitorService monitorService;
    @Autowired
    private CabinetService cabinetService;

    @Test
    void testCabinetCreate() {
        cabinetService.deleteAllCabinets();
        final Cabinet cabinet = cabinetService.addCabinet("18");
        log.info(cabinet.toString());
        Assertions.assertNotNull(cabinet.getId());
    }
    @Test
    void testCabinetRead() {
        cabinetService.deleteAllCabinets();
        final Cabinet cabinet = cabinetService.addCabinet("18");
        log.info(cabinet.toString());
        final Cabinet findCabinet = cabinetService.findCabinet(cabinet.getId());
        log.info(findCabinet.toString());
        Assertions.assertEquals(cabinet, findCabinet);
    }
    @Test
    void testCabinetReadNotFound() {
        cabinetService.deleteAllCabinets();
        Assertions.assertThrows(CabinetNotFoundException.class, () -> cabinetService.findCabinet(-1L));
    }
    @Test
    void testCabinetReadAll() {
        cabinetService.deleteAllCabinets();
        cabinetService.addCabinet("18");
        cabinetService.addCabinet("19");
        final List<Cabinet> cabinets = cabinetService.findAllCabinets();
        log.info(cabinets.toString());
        Assertions.assertEquals(cabinets.size(), 2);
    }
    @Test
    void testComputerReadAllEmpty() {
        cabinetService.deleteAllCabinets();
        final List<Cabinet> cabinets = cabinetService.findAllCabinets();
        log.info(cabinets.toString());
        Assertions.assertEquals(cabinets.size(), 0);
    }
}
