package com.kalyshev.yan;

import com.kalyshev.yan.cabinet.model.Cabinet;
import com.kalyshev.yan.cabinet.service.CabinetService;
import com.kalyshev.yan.computer.model.Computer;
import com.kalyshev.yan.computer.service.ComputerService;
import com.kalyshev.yan.monitor.model.Monitor;
import com.kalyshev.yan.monitor.service.MonitorService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest
public class JpaComputerCabinetTests {
    private static final Logger log = LoggerFactory.getLogger(JpaCabinetTests.class);
    @Autowired
    private ComputerService computerService;
    @Autowired
    private MonitorService monitorService;
    @Autowired
    private CabinetService cabinetService;

    @Test
    void testCabinetAddOneToMany() {
        cabinetService.deleteAllCabinets();
        computerService.deleteAllComputers();
        computerService.deleteAllMonitorsWithRelations();
        final Cabinet cabinet = cabinetService.addCabinet("18");
        final Monitor monitor = monitorService.addMonitor("Asus");
        final Computer computer = computerService.addComputer("PC", "78457h", monitor.getId());
        cabinet.addComputer(computer);
        log.info(cabinet.toString());
        log.info(computer.toString());
        Assertions.assertEquals(cabinet.getComputers().get(0).getSerialNum(), computer.getSerialNum());
    }

    @Test
    void testCabinetDeleteOneToMany() {
        cabinetService.deleteAllCabinets();
        computerService.deleteAllComputers();
        computerService.deleteAllMonitorsWithRelations();
        final Cabinet cabinet = cabinetService.addCabinet("18");
        final Monitor monitor = monitorService.addMonitor("Asus");
        final Computer computer = computerService.addComputer("PC", "78457h", monitor.getId());
        cabinet.addComputer(computer);
        log.info(cabinet.toString());
        log.info(computer.toString());
        Assertions.assertEquals(cabinet.getComputers().get(0).getSerialNum(), computer.getSerialNum());
        cabinet.removeComputer(computer);
        Assertions.assertEquals(cabinet.getComputers().size(), 0);
    }

    @Test
    void testComputerAddManyToMany() {
        cabinetService.deleteAllCabinets();
        computerService.deleteAllComputers();
        computerService.deleteAllMonitorsWithRelations();
        final Cabinet cabinet = cabinetService.addCabinet("18");
        final Monitor monitor = monitorService.addMonitor("Asus");
        final Computer computer = computerService.addComputer("PC", "78457h", monitor.getId());
        computer.setCabinet(cabinet);
        log.info(cabinet.toString());
        log.info(computer.toString());
        Assertions.assertEquals(computer.getCabinet().getNumber(), cabinet.getNumber());
    }

    @Test
    void testComputerDeleteManyToMany() {
        cabinetService.deleteAllCabinets();
        computerService.deleteAllComputers();
        computerService.deleteAllMonitorsWithRelations();
        final Cabinet cabinet = cabinetService.addCabinet("18");
        final Monitor monitor = monitorService.addMonitor("Asus");
        final Computer computer = computerService.addComputer("PC", "78457h", monitor.getId());
        computer.setCabinet(cabinet);
        log.info(cabinet.toString());
        log.info(computer.toString());
        Assertions.assertEquals(computer.getCabinet().getNumber(), cabinet.getNumber());
        computer.setCabinet(null);
        Assertions.assertNull(computer.getCabinet());
    }
}
